import { FetchesJasdActivities } from './dependencies/FetchesJasdActivities.ts'
import { ResolvesCity } from './dependencies/ResolvesCity.ts'
import { HoliEvent } from '../domain/HoliEvent.ts'
import { Logs } from './dependencies/Logs.ts'
import { ActivityResult } from '../adapters/jasd/types/appapi.dto.types.ts'

export interface QueryEventsInput {
  limit: number
  offset: number
  localOnly: boolean
  geolocationId?: string
}

export interface QueryEventsOutput {
  data: HoliEvent[]
  totalResults: number
}

const APP_FILES_BASE_URL = 'https://gemeinschaftswerk-nachhaltigkeit.de/app/api/v1/files'

export class QueryEvents {
  constructor(
    private readonly jasdApi: FetchesJasdActivities,
    private readonly geoApi: ResolvesCity,
    private readonly logger: Logs,
  ) {}

  async execute(input: QueryEventsInput): Promise<QueryEventsOutput> {
    this.logger.debug(`Executing QueryEvents use case with input: ${JSON.stringify(input)}`)
    const startTime = Date.now()

    try {
      const location = input.geolocationId ? await this.geoApi.resolveCityName(input.geolocationId) : undefined

      const jasdEvents = await this.jasdApi.fetchActivities(
        input.limit,
        input.offset,
        input.localOnly,
        location,
      )
      const holiEvents = jasdEvents.events.map(this.toDomainEvent)
      return {
        data: holiEvents,
        totalResults: jasdEvents.totalCount,
      }
    } catch (error) {
      this.logger.error('Error executing QueryEvents use case', error)
      throw error
    } finally {
      const duration = Date.now() - startTime
      this.logger.info(`QueryEvents use case executed in ${duration}ms`)
    }
  }

  private toDomainEvent(event: ActivityResult): HoliEvent {
    return {
      id: event.activity.id.toString(),
      title: event.name,
      organisationName: event.activity.organisation.name,
      startDate: event.activity.period.start,
      endDate: event.activity.period.end,
      location: {
        city: event.location.address.city,
        street: event.location.address.street,
        streetNo: event.location.address.streetNo,
        zipCode: event.location.address.zipCode,
      },
      isRemote: event.activity.location.online,
      imageUrl: `${APP_FILES_BASE_URL}/${event.activity.image}`,
      eventDetails: {
        description: event.description,
        externalLink: event.activity.location.url,
        meetingLink: event.activity.registerUrl ?? undefined,
      },
    }
  }
}
